﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Arction.WPF.LightningChartUltimate;
using Arction.WPF.LightningChartUltimate.Views.View3D;
using Arction.WPF.LightningChartUltimate.Series3D;
using System.Drawing;

namespace LCSurfaceRenderIssue
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        static double _noPassVal = Double.MinValue;
        static double _passRadius = 2.5;
        static double _passRadius2 = _passRadius * _passRadius;

        static double surfaceFunction(double x, double z)
        {
            double r2 = x * x + z * z;
            double funcVal = 0.0;
            if (r2 > _passRadius2)
            {
                funcVal = _noPassVal;
            }
            else
            {
                double expFactor = Math.Exp(-0.75 * r2);
                double cosFactor = Math.Cos(2.0 * r2 / Math.PI);
                cosFactor *= cosFactor;
                funcVal = expFactor*cosFactor;                
            }
            return funcVal;
        }

        private void CalculateButton_Click(object sender, RoutedEventArgs e)
        {
            refreshChart();
        }

        private void refreshChart()
        {
            Chart.ActiveView = Arction.WPF.LightningChartUltimate.ActiveView.View3D;
            Chart.Title.Text = "";
            double minX = -3.0;
            double maxX = 3.0;
            int numX = 32;
            double dx = (maxX - minX) / (double)(numX - 1);
            double minZ = -3.0;
            double maxZ = 3.0;
            int numZ = 32;
            double dz = (maxZ - minZ) / (double)(numZ - 1);

            View3D view = Chart.View3D;
            view.SurfaceGridSeries3D.Clear();

            SurfaceGridSeries3D surface3d = new SurfaceGridSeries3D();

            int numPts = numX * numZ;
            SurfacePoint[,] points = new SurfacePoint[numX, numZ];
            double x, y, z;
            for (int iX = 0; iX < numX; ++iX)
            {
                x = minX + iX * dx;
                for (int iZ = 0; iZ < numZ; ++iZ)
                {
                    z = minZ + iZ * dz;
                    y = surfaceFunction(x, z);

                    SurfacePoint pt = new SurfacePoint();
                    pt.X = x;
                    pt.Y = y;
                    pt.Z = z;

                    points[iX, iZ] = pt;
                }
            }
            surface3d.Data = points;

            ValueRangePalette palette = new ValueRangePalette(surface3d);
            palette.MinValue = 0.0;
            int numPaletteSteps = 11;
            double dPalette = 1.0 / (double)numPaletteSteps;
            for (int iStep = 0; iStep < numPaletteSteps; ++iStep)
            {
                PaletteStep step = new PaletteStep();
                step.MaxValue = dPalette * (iStep + 1);
                step.Color = System.Windows.Media.Color.FromScRgb(1.0f, 1.0f, (float)dPalette * iStep, 0.0f);
                palette.Steps.Add(step);
            }
            surface3d.RangeMinX = minX;
            surface3d.RangeMaxX = maxX;
            surface3d.RangeMinZ = minZ;
            surface3d.RangeMaxZ = maxZ;
            surface3d.ContourPalette = palette;
            surface3d.Fill = SurfaceFillStyle.Paletted;
            surface3d.FastContourZoneHeight = 0.0;
            surface3d.BaseColor = System.Windows.Media.Colors.PaleTurquoise;

            view.XAxisPrimary3D.SetRange(minX, maxX);
            view.YAxisPrimary3D.SetRange(0.0, 1.0);
            view.ZAxisPrimary3D.SetRange(minZ, maxZ);

            view.WallOnBack.Visible = false;
            view.WallOnFront.Visible = false;
            view.WallOnLeft.Visible = false;
            view.WallOnRight.Visible = false;
            view.WallOnTop.Visible = false;
            view.WallOnBottom.Visible = false;
            view.ClipContents = true;

            view.LegendBox.ShowCheckboxes = false;
            int scaleSize1 = 100;
            int scaleSize2 = 50;
            view.LegendBox.SurfaceScales.ScaleSizeDim1 = scaleSize1;
            view.LegendBox.SurfaceScales.ScaleSizeDim2 = scaleSize2;
            view.LegendBox.SurfaceScales.SizeGrid = new System.Drawing.Size(scaleSize1 / 2, scaleSize2 / 2);

            view.SurfaceGridSeries3D.Add(surface3d);
            Chart.InvalidateVisual();
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            Chart.Dispose();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            refreshChart();
        }
    }
}
